const { pool } = require("../config/database/database.config");

async function addPerson(person) {
  // extract all the values from person
  let {
    firstName,
    lastName,
    middleName,
    maidenName,
    suffix,
    nickName,
    techAlumniChapter,
    classYear,
    gradYear,
    gradSemester,
    gender,
  } = person;

  // find the integer that represents the alumni chapter (for now do 2)
  techAlumniChapter = 2;

  console.log("In here now");

  try {
    console.log("Adding person: ", firstName);

    const query = `
      INSERT INTO people (firstName, lastName, middleName, maidenName, suffix, nickName, techAlumniChapter, classYear, gradYear, gradSemester, gender)
      VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    `;

    const [results] = await pool.query(query, [
      firstName,
      lastName,
      middleName,
      maidenName,
      suffix,
      nickName,
      techAlumniChapter,
      classYear,
      gradYear,
      gradSemester,
      gender,
    ]);

    // return the new person's id
    return results.insertId;
  } catch (error) {
    console.error(error);
    return null;
  }
}

/*
Function to search for people by their first and last name, 
if multiple people with that name exist, they will all be returned. 
If no people are found, the function will return null.
*/
async function searchPeopleByName(firstName, lastName) {
  console.log("searching for people by name....");

  try {
    // SQL query to search for people by first and last name
    const query = `
      SELECT * FROM people
      WHERE firstName = ? AND lastName = ?
      `;

    // Execute the query and pass the first and last name as parameters
    const [results] = await pool.query(query, [firstName, lastName]);

    console.log("results found: ", results);

    // If no people are found, return null
    if (results.length === 0) {
      return null;
    }

    // return the people found
    return results;
  } catch (error) {
    console.error(error);
    return null;
  }
}

/*
Function to get the data associated with a specific peopleId.
If none is found, the function returns null.
*/
async function getPersonById(peopleId) {
  console.log("searching for person...");

  try {
    // big query to get all of the data
    const query = `
    SELECT 
      peopleId, 
      firstName, 
      lastName, 
      middleName, 
      maidenName, 
      suffix, 
      nickName, 
      techAlumniChapter, 
      classYear, 
      gradYear, 
      gradSemester, 
      gender
    FROM 
      people
    WHERE 
      peopleId = ?;
  `;

    // Execute the query
    const [rows] = await pool.query(query, [peopleId]);

    if (rows.length === 0) {
      console.log("No person found");
      null;
    }

    // structure the data
    console.log("data recieved... structuring now");

    // create the structure of a person
    const person = rows[0];
    return person;
  } catch (error) {
    console.log("Error getting person's full details: ", error);
    return null;
  }
}

/*
Function to delete a person with the peopleId given. 
Since the SQL schema has ON DELETE CASCADE, all associated data
(address, contacts, etc). will also be deleted. Function returns
true if the delete was successful and false if not.
*/
async function deletePersonById(peopleId) {
  console.log("deleting person by id....");

  try {
    // execute the query to delete by primary key
    const query = `DELETE FROM people WHERE peopleId = ?`;
    const [results] = await pool.query(query, [peopleId]);

    // check if anything was deleted
    if (results.affectedRows === 0) {
      console.log("no rows deleted");
      return false; // no rows deleted
    }

    console.log("row deleted");

    return true;
  } catch (error) {
    console.error(error);
    return false;
  }
}

/*
Function to update the people table's information for
the person with the given id.
*/
async function updatePerson(personId, personInfo) {
  // extract all the values from person
  let {
    firstName,
    lastName,
    middleName,
    maidenName,
    suffix,
    nickName,
    techAlumniChapter,
    classYear,
    gradYear,
    gradSemester,
    gender,
  } = personInfo;

  // find the integer that represents the alumni chapter (for now do 2)
  techAlumniChapter = 2;

  try {
    console.log("Updating person: ", firstName);

    // SQL UPDATE query
    const query = `
     UPDATE people
     SET 
       firstName = ?, 
       lastName = ?, 
       middleName = ?, 
       maidenName = ?, 
       suffix = ?, 
       nickName = ?, 
       techAlumniChapter = ?, 
       classYear = ?, 
       gradYear = ?, 
       gradSemester = ?, 
       gender = ?
     WHERE peopleId = ?
   `;

    const [results] = await pool.query(query, [
      firstName,
      lastName,
      middleName,
      maidenName,
      suffix,
      nickName,
      techAlumniChapter,
      classYear,
      gradYear,
      gradSemester,
      gender,
      personId,
    ]);

    // Check if any rows were affected
    if (results.affectedRows === 0) {
      console.error("Person not found. no rows updated");
      return false;
    }

    console.log("person info updated: ", personId);

    // return true to signify success
    return true;
  } catch (error) {
    console.error(error);
    return false;
  }
}

module.exports = {
  addPerson,
  searchPeopleByName,
  deletePersonById,
  updatePerson,
  getPersonById,
};
