const { pool } = require("../config/database/database.config");

/*
Class responsible for making queries to the 
stateLookup database.
*/

/*
Function to add a state to the database

state must be a string
*/
async function addStateIfNotExists(state) {
  console.log("adding state: ", state);

  // don't add state if it exists
  const existingState = await getState(state);
  if (existingState) {
    console.log("State already exists: ", state);
    return;
  }

  // insert SQL query
  const query = `INSERT INTO stateLookup (name) VALUES (?)`;

  // attempt to insert state into table
  try {
    await pool.query(query, [state]);
    console.log(`Successfully added state: ${state}`);
  } catch (error) {
    console.error(`Error adding state: ${state}`, error);
    throw error;
  }
}

async function getState(name) {
  // SQL query to get a certain state
  const query = "SELECT * FROM stateLookup WHERE name = ?";

  try {
    console.log("Getting state: ", name);
    const [rows] = await pool.query(query, [name]);
    if (rows.length > 0) {
      console.log("Found state: ", name);
      return rows[0];
    } else {
      console.log("State not found: ", name);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching state with ID ${name}:`, error);
    throw error;
  }
}

async function getStateId(name) {
  // SQL query to get a certain state id
  const query = "SELECT stateId FROM stateLookup WHERE name = ?";

  try {
    console.log("Getting state: ", name);
    const [rows] = await pool.query(query, [name]);
    if (rows.length > 0) {
      console.log("Found state: ", name);
      return rows[0];
    } else {
      console.log("State not found: ", name);
      return null;
    }
  } catch (error) {
    console.error(`Error fetching state with ID ${name}:`, error);
    throw error;
  }
}

async function checkTableExistence() {
  try {
    const [rows] = await pool.query("SHOW TABLES LIKE 'stateLookup'");
    if (rows.length > 0) {
      console.log("Table `stateLookup` exists");
    } else {
      console.log("Table `stateLookup` does not exist");
    }
  } catch (error) {
    console.error("Error checking table existence:", error);
  }
}

module.exports = {
  checkTableExistence,
  addStateIfNotExists,
  getState,
  getStateId,
};
